import { useContext, createContext, useState, useMemo, useEffect, forwardRef, useImperativeHandle, useCallback, useRef } from 'react';
import MonacoEditor, { useMonaco } from '@monaco-editor/react';
import { DIAGNOSTIC_SEVERITY, getRange, getHoverInformation, Position, LanguageService, getTokenAtPosition, getAutocompleteSuggestions } from 'graphql-language-service';
export { getRange } from 'graphql-language-service';
import { jsx } from 'react/jsx-runtime';

function devAssert(condition, message) {
  const booleanCondition = Boolean(condition);

  if (!booleanCondition) {
    throw new Error(message);
  }
}

/**
 * Return true if `value` is object-like. A value is object-like if it's not
 * `null` and has a `typeof` result of "object".
 */
function isObjectLike(value) {
  return typeof value == 'object' && value !== null;
}

function invariant(condition, message) {
  const booleanCondition = Boolean(condition);

  if (!booleanCondition) {
    throw new Error(
      message != null ? message : 'Unexpected invariant triggered.',
    );
  }
}

const LineRegExp = /\r\n|[\n\r]/g;
/**
 * Represents a location in a Source.
 */

/**
 * Takes a Source and a UTF-8 character offset, and returns the corresponding
 * line and column as a SourceLocation.
 */
function getLocation(source, position) {
  let lastLineStart = 0;
  let line = 1;

  for (const match of source.body.matchAll(LineRegExp)) {
    typeof match.index === 'number' || invariant(false);

    if (match.index >= position) {
      break;
    }

    lastLineStart = match.index + match[0].length;
    line += 1;
  }

  return {
    line,
    column: position + 1 - lastLineStart,
  };
}

/**
 * Render a helpful description of the location in the GraphQL Source document.
 */
function printLocation(location) {
  return printSourceLocation(
    location.source,
    getLocation(location.source, location.start),
  );
}
/**
 * Render a helpful description of the location in the GraphQL Source document.
 */

function printSourceLocation(source, sourceLocation) {
  const firstLineColumnOffset = source.locationOffset.column - 1;
  const body = ''.padStart(firstLineColumnOffset) + source.body;
  const lineIndex = sourceLocation.line - 1;
  const lineOffset = source.locationOffset.line - 1;
  const lineNum = sourceLocation.line + lineOffset;
  const columnOffset = sourceLocation.line === 1 ? firstLineColumnOffset : 0;
  const columnNum = sourceLocation.column + columnOffset;
  const locationStr = `${source.name}:${lineNum}:${columnNum}\n`;
  const lines = body.split(/\r\n|[\n\r]/g);
  const locationLine = lines[lineIndex]; // Special case for minified documents

  if (locationLine.length > 120) {
    const subLineIndex = Math.floor(columnNum / 80);
    const subLineColumnNum = columnNum % 80;
    const subLines = [];

    for (let i = 0; i < locationLine.length; i += 80) {
      subLines.push(locationLine.slice(i, i + 80));
    }

    return (
      locationStr +
      printPrefixedLines([
        [`${lineNum} |`, subLines[0]],
        ...subLines.slice(1, subLineIndex + 1).map((subLine) => ['|', subLine]),
        ['|', '^'.padStart(subLineColumnNum)],
        ['|', subLines[subLineIndex + 1]],
      ])
    );
  }

  return (
    locationStr +
    printPrefixedLines([
      // Lines specified like this: ["prefix", "string"],
      [`${lineNum - 1} |`, lines[lineIndex - 1]],
      [`${lineNum} |`, locationLine],
      ['|', '^'.padStart(columnNum)],
      [`${lineNum + 1} |`, lines[lineIndex + 1]],
    ])
  );
}

function printPrefixedLines(lines) {
  const existingLines = lines.filter(([_, line]) => line !== undefined);
  const padLen = Math.max(...existingLines.map(([prefix]) => prefix.length));
  return existingLines
    .map(([prefix, line]) => prefix.padStart(padLen) + (line ? ' ' + line : ''))
    .join('\n');
}

function toNormalizedOptions(args) {
  const firstArg = args[0];

  if (firstArg == null || 'kind' in firstArg || 'length' in firstArg) {
    return {
      nodes: firstArg,
      source: args[1],
      positions: args[2],
      path: args[3],
      originalError: args[4],
      extensions: args[5],
    };
  }

  return firstArg;
}
/**
 * A GraphQLError describes an Error found during the parse, validate, or
 * execute phases of performing a GraphQL operation. In addition to a message
 * and stack trace, it also includes information about the locations in a
 * GraphQL document and/or execution result that correspond to the Error.
 */

class GraphQLError extends Error {
  /**
   * An array of `{ line, column }` locations within the source GraphQL document
   * which correspond to this error.
   *
   * Errors during validation often contain multiple locations, for example to
   * point out two things with the same name. Errors during execution include a
   * single location, the field which produced the error.
   *
   * Enumerable, and appears in the result of JSON.stringify().
   */

  /**
   * An array describing the JSON-path into the execution response which
   * corresponds to this error. Only included for errors during execution.
   *
   * Enumerable, and appears in the result of JSON.stringify().
   */

  /**
   * An array of GraphQL AST Nodes corresponding to this error.
   */

  /**
   * The source GraphQL document for the first location of this error.
   *
   * Note that if this Error represents more than one node, the source may not
   * represent nodes after the first node.
   */

  /**
   * An array of character offsets within the source GraphQL document
   * which correspond to this error.
   */

  /**
   * The original error thrown from a field resolver during execution.
   */

  /**
   * Extension fields to add to the formatted error.
   */

  /**
   * @deprecated Please use the `GraphQLErrorOptions` constructor overload instead.
   */
  constructor(message, ...rawArgs) {
    var _this$nodes, _nodeLocations$, _ref;

    const { nodes, source, positions, path, originalError, extensions } =
      toNormalizedOptions(rawArgs);
    super(message);
    this.name = 'GraphQLError';
    this.path = path !== null && path !== void 0 ? path : undefined;
    this.originalError =
      originalError !== null && originalError !== void 0
        ? originalError
        : undefined; // Compute list of blame nodes.

    this.nodes = undefinedIfEmpty(
      Array.isArray(nodes) ? nodes : nodes ? [nodes] : undefined,
    );
    const nodeLocations = undefinedIfEmpty(
      (_this$nodes = this.nodes) === null || _this$nodes === void 0
        ? void 0
        : _this$nodes.map((node) => node.loc).filter((loc) => loc != null),
    ); // Compute locations in the source for the given nodes/positions.

    this.source =
      source !== null && source !== void 0
        ? source
        : nodeLocations === null || nodeLocations === void 0
        ? void 0
        : (_nodeLocations$ = nodeLocations[0]) === null ||
          _nodeLocations$ === void 0
        ? void 0
        : _nodeLocations$.source;
    this.positions =
      positions !== null && positions !== void 0
        ? positions
        : nodeLocations === null || nodeLocations === void 0
        ? void 0
        : nodeLocations.map((loc) => loc.start);
    this.locations =
      positions && source
        ? positions.map((pos) => getLocation(source, pos))
        : nodeLocations === null || nodeLocations === void 0
        ? void 0
        : nodeLocations.map((loc) => getLocation(loc.source, loc.start));
    const originalExtensions = isObjectLike(
      originalError === null || originalError === void 0
        ? void 0
        : originalError.extensions,
    )
      ? originalError === null || originalError === void 0
        ? void 0
        : originalError.extensions
      : undefined;
    this.extensions =
      (_ref =
        extensions !== null && extensions !== void 0
          ? extensions
          : originalExtensions) !== null && _ref !== void 0
        ? _ref
        : Object.create(null); // Only properties prescribed by the spec should be enumerable.
    // Keep the rest as non-enumerable.

    Object.defineProperties(this, {
      message: {
        writable: true,
        enumerable: true,
      },
      name: {
        enumerable: false,
      },
      nodes: {
        enumerable: false,
      },
      source: {
        enumerable: false,
      },
      positions: {
        enumerable: false,
      },
      originalError: {
        enumerable: false,
      },
    }); // Include (non-enumerable) stack trace.

    /* c8 ignore start */
    // FIXME: https://github.com/graphql/graphql-js/issues/2317

    if (
      originalError !== null &&
      originalError !== void 0 &&
      originalError.stack
    ) {
      Object.defineProperty(this, 'stack', {
        value: originalError.stack,
        writable: true,
        configurable: true,
      });
    } else if (Error.captureStackTrace) {
      Error.captureStackTrace(this, GraphQLError);
    } else {
      Object.defineProperty(this, 'stack', {
        value: Error().stack,
        writable: true,
        configurable: true,
      });
    }
    /* c8 ignore stop */
  }

  get [Symbol.toStringTag]() {
    return 'GraphQLError';
  }

  toString() {
    let output = this.message;

    if (this.nodes) {
      for (const node of this.nodes) {
        if (node.loc) {
          output += '\n\n' + printLocation(node.loc);
        }
      }
    } else if (this.source && this.locations) {
      for (const location of this.locations) {
        output += '\n\n' + printSourceLocation(this.source, location);
      }
    }

    return output;
  }

  toJSON() {
    const formattedError = {
      message: this.message,
    };

    if (this.locations != null) {
      formattedError.locations = this.locations;
    }

    if (this.path != null) {
      formattedError.path = this.path;
    }

    if (this.extensions != null && Object.keys(this.extensions).length > 0) {
      formattedError.extensions = this.extensions;
    }

    return formattedError;
  }
}

function undefinedIfEmpty(array) {
  return array === undefined || array.length === 0 ? undefined : array;
}

/**
 * ```
 * WhiteSpace ::
 *   - "Horizontal Tab (U+0009)"
 *   - "Space (U+0020)"
 * ```
 * @internal
 */
/**
 * ```
 * Digit :: one of
 *   - `0` `1` `2` `3` `4` `5` `6` `7` `8` `9`
 * ```
 * @internal
 */

function isDigit(code) {
  return code >= 0x0030 && code <= 0x0039;
}
/**
 * ```
 * Letter :: one of
 *   - `A` `B` `C` `D` `E` `F` `G` `H` `I` `J` `K` `L` `M`
 *   - `N` `O` `P` `Q` `R` `S` `T` `U` `V` `W` `X` `Y` `Z`
 *   - `a` `b` `c` `d` `e` `f` `g` `h` `i` `j` `k` `l` `m`
 *   - `n` `o` `p` `q` `r` `s` `t` `u` `v` `w` `x` `y` `z`
 * ```
 * @internal
 */

function isLetter(code) {
  return (
    (code >= 0x0061 && code <= 0x007a) || // A-Z
    (code >= 0x0041 && code <= 0x005a) // a-z
  );
}
/**
 * ```
 * NameStart ::
 *   - Letter
 *   - `_`
 * ```
 * @internal
 */

function isNameStart(code) {
  return isLetter(code) || code === 0x005f;
}
/**
 * ```
 * NameContinue ::
 *   - Letter
 *   - Digit
 *   - `_`
 * ```
 * @internal
 */

function isNameContinue(code) {
  return isLetter(code) || isDigit(code) || code === 0x005f;
}

const MAX_ARRAY_LENGTH = 10;
const MAX_RECURSIVE_DEPTH = 2;
/**
 * Used to print values in error messages.
 */

function inspect(value) {
  return formatValue(value, []);
}

function formatValue(value, seenValues) {
  switch (typeof value) {
    case 'string':
      return JSON.stringify(value);

    case 'function':
      return value.name ? `[function ${value.name}]` : '[function]';

    case 'object':
      return formatObjectValue(value, seenValues);

    default:
      return String(value);
  }
}

function formatObjectValue(value, previouslySeenValues) {
  if (value === null) {
    return 'null';
  }

  if (previouslySeenValues.includes(value)) {
    return '[Circular]';
  }

  const seenValues = [...previouslySeenValues, value];

  if (isJSONable(value)) {
    const jsonValue = value.toJSON(); // check for infinite recursion

    if (jsonValue !== value) {
      return typeof jsonValue === 'string'
        ? jsonValue
        : formatValue(jsonValue, seenValues);
    }
  } else if (Array.isArray(value)) {
    return formatArray(value, seenValues);
  }

  return formatObject(value, seenValues);
}

function isJSONable(value) {
  return typeof value.toJSON === 'function';
}

function formatObject(object, seenValues) {
  const entries = Object.entries(object);

  if (entries.length === 0) {
    return '{}';
  }

  if (seenValues.length > MAX_RECURSIVE_DEPTH) {
    return '[' + getObjectTag(object) + ']';
  }

  const properties = entries.map(
    ([key, value]) => key + ': ' + formatValue(value, seenValues),
  );
  return '{ ' + properties.join(', ') + ' }';
}

function formatArray(array, seenValues) {
  if (array.length === 0) {
    return '[]';
  }

  if (seenValues.length > MAX_RECURSIVE_DEPTH) {
    return '[Array]';
  }

  const len = Math.min(MAX_ARRAY_LENGTH, array.length);
  const remaining = array.length - len;
  const items = [];

  for (let i = 0; i < len; ++i) {
    items.push(formatValue(array[i], seenValues));
  }

  if (remaining === 1) {
    items.push('... 1 more item');
  } else if (remaining > 1) {
    items.push(`... ${remaining} more items`);
  }

  return '[' + items.join(', ') + ']';
}

function getObjectTag(object) {
  const tag = Object.prototype.toString
    .call(object)
    .replace(/^\[object /, '')
    .replace(/]$/, '');

  if (tag === 'Object' && typeof object.constructor === 'function') {
    const name = object.constructor.name;

    if (typeof name === 'string' && name !== '') {
      return name;
    }
  }

  return tag;
}

/**
 * A replacement for instanceof which includes an error warning when multi-realm
 * constructors are detected.
 * See: https://expressjs.com/en/advanced/best-practice-performance.html#set-node_env-to-production
 * See: https://webpack.js.org/guides/production/
 */

const instanceOf =
  /* c8 ignore next 6 */
  // FIXME: https://github.com/graphql/graphql-js/issues/2317
  // eslint-disable-next-line no-undef
  process.env.NODE_ENV === 'production'
    ? function instanceOf(value, constructor) {
        return value instanceof constructor;
      }
    : function instanceOf(value, constructor) {
        if (value instanceof constructor) {
          return true;
        }

        if (typeof value === 'object' && value !== null) {
          var _value$constructor;

          // Prefer Symbol.toStringTag since it is immune to minification.
          const className = constructor.prototype[Symbol.toStringTag];
          const valueClassName = // We still need to support constructor's name to detect conflicts with older versions of this library.
            Symbol.toStringTag in value // @ts-expect-error TS bug see, https://github.com/microsoft/TypeScript/issues/38009
              ? value[Symbol.toStringTag]
              : (_value$constructor = value.constructor) === null ||
                _value$constructor === void 0
              ? void 0
              : _value$constructor.name;

          if (className === valueClassName) {
            const stringifiedValue = inspect(value);
            throw new Error(`Cannot use ${className} "${stringifiedValue}" from another module or realm.

Ensure that there is only one instance of "graphql" in the node_modules
directory. If different versions of "graphql" are the dependencies of other
relied on modules, use "resolutions" to ensure only one version is installed.

https://yarnpkg.com/en/docs/selective-version-resolutions

Duplicate "graphql" modules cannot be used at the same time since different
versions may have different capabilities and behavior. The data from one
version used in the function from another could produce confusing and
spurious results.`);
          }
        }

        return false;
      };

/**
 * Creates a keyed JS object from an array, given a function to produce the keys
 * and a function to produce the values from each item in the array.
 * ```ts
 * const phoneBook = [
 *   { name: 'Jon', num: '555-1234' },
 *   { name: 'Jenny', num: '867-5309' }
 * ]
 *
 * // { Jon: '555-1234', Jenny: '867-5309' }
 * const phonesByName = keyValMap(
 *   phoneBook,
 *   entry => entry.name,
 *   entry => entry.num
 * )
 * ```
 */
function keyValMap(list, keyFn, valFn) {
  const result = Object.create(null);

  for (const item of list) {
    result[keyFn(item)] = valFn(item);
  }

  return result;
}

/**
 * Creates an object map with the same keys as `map` and values generated by
 * running each value of `map` thru `fn`.
 */
function mapValue(map, fn) {
  const result = Object.create(null);

  for (const key of Object.keys(map)) {
    result[key] = fn(map[key], key);
  }

  return result;
}

function toObjMap(obj) {
  if (obj == null) {
    return Object.create(null);
  }

  if (Object.getPrototypeOf(obj) === null) {
    return obj;
  }

  const map = Object.create(null);

  for (const [key, value] of Object.entries(obj)) {
    map[key] = value;
  }

  return map;
}

/**
 * Upholds the spec rules about naming.
 */

function assertName(name) {
  name != null || devAssert(false, 'Must provide name.');
  typeof name === 'string' || devAssert(false, 'Expected name to be a string.');

  if (name.length === 0) {
    throw new GraphQLError('Expected name to be a non-empty string.');
  }

  for (let i = 1; i < name.length; ++i) {
    if (!isNameContinue(name.charCodeAt(i))) {
      throw new GraphQLError(
        `Names must only contain [_a-zA-Z0-9] but "${name}" does not.`,
      );
    }
  }

  if (!isNameStart(name.charCodeAt(0))) {
    throw new GraphQLError(
      `Names must start with [_a-zA-Z] but "${name}" does not.`,
    );
  }

  return name;
}

function isObjectType(type) {
  return instanceOf(type, GraphQLObjectType);
}
function isInterfaceType(type) {
  return instanceOf(type, GraphQLInterfaceType);
}
/**
 * Used while defining GraphQL types to allow for circular references in
 * otherwise immutable type definitions.
 */

function resolveReadonlyArrayThunk(thunk) {
  return typeof thunk === 'function' ? thunk() : thunk;
}
function resolveObjMapThunk(thunk) {
  return typeof thunk === 'function' ? thunk() : thunk;
}

/**
 * Object Type Definition
 *
 * Almost all of the GraphQL types you define will be object types. Object types
 * have a name, but most importantly describe their fields.
 *
 * Example:
 *
 * ```ts
 * const AddressType = new GraphQLObjectType({
 *   name: 'Address',
 *   fields: {
 *     street: { type: GraphQLString },
 *     number: { type: GraphQLInt },
 *     formatted: {
 *       type: GraphQLString,
 *       resolve(obj) {
 *         return obj.number + ' ' + obj.street
 *       }
 *     }
 *   }
 * });
 * ```
 *
 * When two types need to refer to each other, or a type needs to refer to
 * itself in a field, you can use a function expression (aka a closure or a
 * thunk) to supply the fields lazily.
 *
 * Example:
 *
 * ```ts
 * const PersonType = new GraphQLObjectType({
 *   name: 'Person',
 *   fields: () => ({
 *     name: { type: GraphQLString },
 *     bestFriend: { type: PersonType },
 *   })
 * });
 * ```
 */
class GraphQLObjectType {
  constructor(config) {
    var _config$extensionASTN2;

    this.name = assertName(config.name);
    this.description = config.description;
    this.isTypeOf = config.isTypeOf;
    this.extensions = toObjMap(config.extensions);
    this.astNode = config.astNode;
    this.extensionASTNodes =
      (_config$extensionASTN2 = config.extensionASTNodes) !== null &&
      _config$extensionASTN2 !== void 0
        ? _config$extensionASTN2
        : [];

    this._fields = () => defineFieldMap(config);

    this._interfaces = () => defineInterfaces(config);

    config.isTypeOf == null ||
      typeof config.isTypeOf === 'function' ||
      devAssert(
        false,
        `${this.name} must provide "isTypeOf" as a function, ` +
          `but got: ${inspect(config.isTypeOf)}.`,
      );
  }

  get [Symbol.toStringTag]() {
    return 'GraphQLObjectType';
  }

  getFields() {
    if (typeof this._fields === 'function') {
      this._fields = this._fields();
    }

    return this._fields;
  }

  getInterfaces() {
    if (typeof this._interfaces === 'function') {
      this._interfaces = this._interfaces();
    }

    return this._interfaces;
  }

  toConfig() {
    return {
      name: this.name,
      description: this.description,
      interfaces: this.getInterfaces(),
      fields: fieldsToFieldsConfig(this.getFields()),
      isTypeOf: this.isTypeOf,
      extensions: this.extensions,
      astNode: this.astNode,
      extensionASTNodes: this.extensionASTNodes,
    };
  }

  toString() {
    return this.name;
  }

  toJSON() {
    return this.toString();
  }
}

function defineInterfaces(config) {
  var _config$interfaces;

  const interfaces = resolveReadonlyArrayThunk(
    (_config$interfaces = config.interfaces) !== null &&
      _config$interfaces !== void 0
      ? _config$interfaces
      : [],
  );
  Array.isArray(interfaces) ||
    devAssert(
      false,
      `${config.name} interfaces must be an Array or a function which returns an Array.`,
    );
  return interfaces;
}

function defineFieldMap(config) {
  const fieldMap = resolveObjMapThunk(config.fields);
  isPlainObj(fieldMap) ||
    devAssert(
      false,
      `${config.name} fields must be an object with field names as keys or a function which returns such an object.`,
    );
  return mapValue(fieldMap, (fieldConfig, fieldName) => {
    var _fieldConfig$args;

    isPlainObj(fieldConfig) ||
      devAssert(
        false,
        `${config.name}.${fieldName} field config must be an object.`,
      );
    fieldConfig.resolve == null ||
      typeof fieldConfig.resolve === 'function' ||
      devAssert(
        false,
        `${config.name}.${fieldName} field resolver must be a function if ` +
          `provided, but got: ${inspect(fieldConfig.resolve)}.`,
      );
    const argsConfig =
      (_fieldConfig$args = fieldConfig.args) !== null &&
      _fieldConfig$args !== void 0
        ? _fieldConfig$args
        : {};
    isPlainObj(argsConfig) ||
      devAssert(
        false,
        `${config.name}.${fieldName} args must be an object with argument names as keys.`,
      );
    return {
      name: assertName(fieldName),
      description: fieldConfig.description,
      type: fieldConfig.type,
      args: defineArguments(argsConfig),
      resolve: fieldConfig.resolve,
      subscribe: fieldConfig.subscribe,
      deprecationReason: fieldConfig.deprecationReason,
      extensions: toObjMap(fieldConfig.extensions),
      astNode: fieldConfig.astNode,
    };
  });
}

function defineArguments(config) {
  return Object.entries(config).map(([argName, argConfig]) => ({
    name: assertName(argName),
    description: argConfig.description,
    type: argConfig.type,
    defaultValue: argConfig.defaultValue,
    deprecationReason: argConfig.deprecationReason,
    extensions: toObjMap(argConfig.extensions),
    astNode: argConfig.astNode,
  }));
}

function isPlainObj(obj) {
  return isObjectLike(obj) && !Array.isArray(obj);
}

function fieldsToFieldsConfig(fields) {
  return mapValue(fields, (field) => ({
    description: field.description,
    type: field.type,
    args: argsToArgsConfig(field.args),
    resolve: field.resolve,
    subscribe: field.subscribe,
    deprecationReason: field.deprecationReason,
    extensions: field.extensions,
    astNode: field.astNode,
  }));
}
/**
 * @internal
 */

function argsToArgsConfig(args) {
  return keyValMap(
    args,
    (arg) => arg.name,
    (arg) => ({
      description: arg.description,
      type: arg.type,
      defaultValue: arg.defaultValue,
      deprecationReason: arg.deprecationReason,
      extensions: arg.extensions,
      astNode: arg.astNode,
    }),
  );
}

/**
 * Interface Type Definition
 *
 * When a field can return one of a heterogeneous set of types, a Interface type
 * is used to describe what types are possible, what fields are in common across
 * all types, as well as a function to determine which type is actually used
 * when the field is resolved.
 *
 * Example:
 *
 * ```ts
 * const EntityType = new GraphQLInterfaceType({
 *   name: 'Entity',
 *   fields: {
 *     name: { type: GraphQLString }
 *   }
 * });
 * ```
 */
class GraphQLInterfaceType {
  constructor(config) {
    var _config$extensionASTN3;

    this.name = assertName(config.name);
    this.description = config.description;
    this.resolveType = config.resolveType;
    this.extensions = toObjMap(config.extensions);
    this.astNode = config.astNode;
    this.extensionASTNodes =
      (_config$extensionASTN3 = config.extensionASTNodes) !== null &&
      _config$extensionASTN3 !== void 0
        ? _config$extensionASTN3
        : [];
    this._fields = defineFieldMap.bind(undefined, config);
    this._interfaces = defineInterfaces.bind(undefined, config);
    config.resolveType == null ||
      typeof config.resolveType === 'function' ||
      devAssert(
        false,
        `${this.name} must provide "resolveType" as a function, ` +
          `but got: ${inspect(config.resolveType)}.`,
      );
  }

  get [Symbol.toStringTag]() {
    return 'GraphQLInterfaceType';
  }

  getFields() {
    if (typeof this._fields === 'function') {
      this._fields = this._fields();
    }

    return this._fields;
  }

  getInterfaces() {
    if (typeof this._interfaces === 'function') {
      this._interfaces = this._interfaces();
    }

    return this._interfaces;
  }

  toConfig() {
    return {
      name: this.name,
      description: this.description,
      interfaces: this.getInterfaces(),
      fields: fieldsToFieldsConfig(this.getFields()),
      resolveType: this.resolveType,
      extensions: this.extensions,
      astNode: this.astNode,
      extensionASTNodes: this.extensionASTNodes,
    };
  }

  toString() {
    return this.name;
  }

  toJSON() {
    return this.toString();
  }
}

const d=/*#__PURE__*/createContext(void 0),u={setTheme:e=>{},themes:[]},h=()=>{var e;return null!==(e=useContext(d))&&void 0!==e?e:u};

function removeFalsey(obj) {
  return Boolean(obj);
}
function locToRange(loc) {
  return {
    startLineNumber: loc.startToken.line,
    startColumn: loc.startToken.column,
    endLineNumber: loc.endToken.line + 1,
    // Because GraphQL starts with 0, and Monaco starts with 1
    endColumn: loc.endToken.column
  };
}
const emptyLocation = {
  startLineNumber: 0,
  startColumn: 0,
  endLineNumber: 0,
  endColumn: 0
};
const coreDiagnosticsSource = {
  async forDocument(_ref) {
    let {
      model,
      document,
      languageService
    } = _ref;
    return languageService.getDiagnostics(model.uri.toString(), document).then(diag => diag.map(toMarkerData)).catch(e => {
      if ('message' in e && 'locations' in e) {
        return [toMarkerData({
          severity: DIAGNOSTIC_SEVERITY.Error,
          message: e.message,
          source: 'GraphQL: Syntax',
          range: getRange(e.locations[0], document)
        })];
      } else {
        console.warn(`GraphQL getDiagnostics failed unexpected error: `, e);
        return [];
      }
    });
  }

};
const coreDefinitionSource = {
  forNode: _ref2 => {
    let {
      schema,
      model,
      token
    } = _ref2;

    if (token.state && token.state.kind === 'NamedType' && token.state.name) {
      const type = schema.getType(token.state.name);

      if (type && type.astNode && type.astNode.loc) {
        return [{
          range: {
            startLineNumber: type.astNode.loc.startToken.line,
            startColumn: type.astNode.loc.startToken.column,
            endLineNumber: type.astNode.loc.endToken.line + 1,
            endColumn: type.astNode.loc.endToken.column
          },
          uri: model.uri
        }];
      }
    }

    return [];
  }
};
const coreHoverSource = {
  forNode: _ref3 => {
    let {
      schema,
      document,
      position,
      token
    } = _ref3;
    return {
      value: getHoverInformation(schema, document, position, token)
    };
  }
};
const debugHoverSource = {
  forNode: _ref4 => {
    let {
      token
    } = _ref4;
    return {
      value: `\`\`\`json\n${JSON.stringify(token.state, null, 2)}\n\`\`\``
    };
  }
};
function toGraphQLPosition(position) {
  return new Position(position.lineNumber - 1, position.column - 1);
}
function toMarkerData(diagnostic) {
  return {
    startLineNumber: diagnostic.range.start.line + 1,
    endLineNumber: diagnostic.range.end.line + 1,
    startColumn: diagnostic.range.start.character + 1,
    endColumn: diagnostic.range.end.character,
    message: diagnostic.message,
    severity: 5,
    // severity: toMonacoSeverity(diagnostic.severity),
    code: diagnostic.code || undefined
  };
}
function toMonacoRange(range) {
  return {
    startLineNumber: range.start.line + 1,
    startColumn: range.start.character + 1,
    endLineNumber: range.end.line + 1,
    endColumn: range.end.character + 1
  };
}
function showWidgetInPosition(editorInstance, position, htmlElement) {
  editorInstance.changeViewZones(function (changeAccessor) {
    changeAccessor.addZone({
      afterLineNumber: position.line + 1,
      heightInPx: 60,
      domNode: htmlElement
    });
  });
}

class EnrichedLanguageService extends LanguageService {
  async getNodeAtPosition(schema, document, position) {
    if (schema) {
      const token = getTokenAtPosition(document, position);

      if (token) {
        return token;
      }
    }

    return null;
  }

  async buildBridgeForProviders(model, position) {
    const graphQLPosition = toGraphQLPosition(position);
    const document = model.getValue();
    const schema = await this.getSchema();

    if (!schema) {
      return null;
    }

    const tokenAtPosition = await this.getNodeAtPosition(schema, document, graphQLPosition);

    if (!tokenAtPosition) {
      return null;
    }

    return {
      languageService: this,
      position: graphQLPosition,
      document,
      schema,
      token: tokenAtPosition,
      model
    };
  }

  getDefinitionProvider(rawSources) {
    const sources = [...rawSources, coreDefinitionSource];
    return {
      provideDefinition: async (model, position) => {
        const bridge = await this.buildBridgeForProviders(model, position);

        if (!bridge) {
          return [];
        }

        const nestedArrays = (await Promise.all(sources.map(source => source.forNode(bridge)))).filter(Boolean);
        const items = [].concat(...nestedArrays);
        return items;
      }
    };
  }

  getHoverProvider(rawSources) {
    const sources = [...rawSources, coreHoverSource];
    return {
      provideHover: async (model, position) => {
        const info = await this.buildBridgeForProviders(model, position);

        if (!info) {
          return;
        }

        const contents = await Promise.all(sources.map(async source => {
          try {
            return await source.forNode(info);
          } catch (e) {
            return null;
          }
        }));
        return {
          contents: contents.filter(removeFalsey),
          range: toMonacoRange(getRange({
            column: info.position.character,
            line: info.position.line + 1
          }, info.document))
        };
      }
    };
  }

  async handleDecorations(decorationSources, model, monacoInstance, editorInstance) {
    for (const source of decorationSources) {
      source.forDocument({
        monaco: monacoInstance,
        editor: editorInstance,
        document: model.getValue(),
        model,
        languageService: this
      });
    }
  }

  async handleDiagnostics(rawDiagnosticsSources, model, monacoInstance) {
    const diagnosticsSources = [...rawDiagnosticsSources, coreDiagnosticsSource];
    const nestedArrays = (await Promise.all(diagnosticsSources.map(async source => {
      try {
        return await source.forDocument({
          languageService: this,
          model,
          document: model.getValue().toString()
        });
      } catch (e) {
        return null;
      }
    }))).filter(removeFalsey);
    const markerData = [].concat(...nestedArrays);
    monacoInstance.editor.setModelMarkers(model, 'graphql', markerData);
  }

  getModelChangeHandler() {
    return async (editorInstance, monacoInstance, diagnosticsSources, decorationsSources) => {
      const model = editorInstance.getModel();

      if (!model) {
        return;
      }

      await Promise.all([this.handleDiagnostics(diagnosticsSources, model, monacoInstance), this.handleDecorations(decorationsSources, model, monacoInstance, editorInstance)]);
    };
  }

  trySchema(sdl) {
    // XX-DOTAN: In case of an issue with the schema parsing, we'll get the error through
    // the Marker, and we can safely ignore it here.
    return this.setSchema(sdl).then(() => this.getSchema());
  }

}

const useSchemaServices = function () {
  let options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  const [editorRef, setEditor] = useState(null);
  const [monacoRef, setMonaco] = useState(null);
  const languageService = useMemo(() => options.sharedLanguageService || new EnrichedLanguageService({
    schemaString: options.schema,
    schemaConfig: {
      buildSchemaOptions: {
        assumeValid: true,
        assumeValidSDL: true
      }
    }
  }), [options.sharedLanguageService]);
  useEffect(() => {
    if (monacoRef && editorRef) {
      if (options.keyboardShortcuts) {
        for (const action of options.keyboardShortcuts(editorRef, monacoRef)) {
          editorRef.addAction(action);
        }
      }

      for (const action of options.actions || []) {
        editorRef.addAction({
          id: action.id,
          label: action.label,
          keybindings: action.keybindings,
          contextMenuGroupId: action.contextMenuGroupId || 'navigation',
          contextMenuOrder: action.contextMenuOrder,

          async run(editor) {
            const model = editor.getModel();
            const position = editor.getPosition();

            if (model && position) {
              const bridge = await languageService.buildBridgeForProviders(model, position);

              if (bridge) {
                action.onRun({
                  editor: editorRef,
                  monaco: monacoRef,
                  bridge
                });
              }
            }
          }

        });
      }

      const handler = languageService.getModelChangeHandler();
      handler(editorRef, monacoRef, options.diagnosticsProviders || [], options.decorationsProviders || []);
      const onChangeDisposable = editorRef.onDidChangeModelContent(() => handler(editorRef, monacoRef, options.diagnosticsProviders || [], options.decorationsProviders || []));
      const definitionProviderDisposable = monacoRef.languages.registerDefinitionProvider('graphql', languageService.getDefinitionProvider(options.definitionProviders || []));
      const hoverDisposable = monacoRef.languages.registerHoverProvider('graphql', languageService.getHoverProvider(options.hoverProviders || []));
      return () => {
        hoverDisposable === null || hoverDisposable === void 0 ? void 0 : hoverDisposable.dispose();
        definitionProviderDisposable === null || definitionProviderDisposable === void 0 ? void 0 : definitionProviderDisposable.dispose();
        onChangeDisposable === null || onChangeDisposable === void 0 ? void 0 : onChangeDisposable.dispose();
      };
    } // eslint-disable-next-line @typescript-eslint/no-empty-function


    return () => {};
  }, [editorRef, monacoRef]);
  return {
    setEditor,
    setMonaco,
    editorRef,
    monacoRef,
    languageService,
    setSchema: newValue => languageService.trySchema(newValue),
    editorApi: {
      jumpToType: typeName => {
        languageService.getSchema().then(schema => {
          if (schema) {
            var _type$astNode;

            const type = schema.getType(typeName);

            if (type !== null && type !== void 0 && (_type$astNode = type.astNode) !== null && _type$astNode !== void 0 && _type$astNode.loc) {
              const range = locToRange(type.astNode.loc);
              editorRef === null || editorRef === void 0 ? void 0 : editorRef.setSelection(range);
              editorRef === null || editorRef === void 0 ? void 0 : editorRef.revealPositionInCenter({
                column: 0,
                lineNumber: range.startLineNumber
              }, 0);
            }
          }
        });
      },

      jumpToField(typeName, fieldName) {
        languageService.getSchema().then(schema => {
          if (schema) {
            const type = schema.getType(typeName);

            if (type && (isObjectType(type) || isInterfaceType(type))) {
              var _field$astNode;

              const field = type.getFields()[fieldName];

              if (field !== null && field !== void 0 && (_field$astNode = field.astNode) !== null && _field$astNode !== void 0 && _field$astNode.loc) {
                const range = locToRange(field.astNode.loc);
                editorRef === null || editorRef === void 0 ? void 0 : editorRef.setSelection(range);
                editorRef === null || editorRef === void 0 ? void 0 : editorRef.revealPositionInCenter({
                  column: 0,
                  lineNumber: range.startLineNumber
                }, 0);
              }
            }
          }
        });
      },

      deselect: () => editorRef === null || editorRef === void 0 ? void 0 : editorRef.setSelection(emptyLocation)
    }
  };
};

function BaseSchemaEditor(props, ref) {
  const {
    theme
  } = h();
  const {
    languageService,
    setMonaco,
    setEditor,
    editorApi,
    editorRef,
    setSchema
  } = useSchemaServices(props);
  useImperativeHandle(ref, () => editorApi, [editorRef, languageService]);
  useEffect(() => {
    if (languageService && props.onLanguageServiceReady) {
      props.onLanguageServiceReady(languageService);
    }
  }, [languageService, props.onLanguageServiceReady]);
  const [onBlurHandler, setOnBlurSubscription] = useState();
  useEffect(() => {
    if (editorRef && props.onBlur) {
      onBlurHandler === null || onBlurHandler === void 0 ? void 0 : onBlurHandler.dispose();
      const subscription = editorRef.onDidBlurEditorText(() => {
        var _props$onBlur;

        (_props$onBlur = props.onBlur) === null || _props$onBlur === void 0 ? void 0 : _props$onBlur.call(props, editorRef.getValue() || '');
      });
      setOnBlurSubscription(subscription);
    }
  }, [props.onBlur, editorRef]);
  const handleBeforeMount = useCallback(monaco => {
    var _props$beforeMount;

    setMonaco(monaco);
    (_props$beforeMount = props.beforeMount) === null || _props$beforeMount === void 0 ? void 0 : _props$beforeMount.call(props, monaco);
  }, [props.beforeMount]);
  const handleMount = useCallback((editor, monaco) => {
    var _props$onMount;

    setEditor(editor);
    (_props$onMount = props.onMount) === null || _props$onMount === void 0 ? void 0 : _props$onMount.call(props, editor, monaco);
  }, [props.onMount]);
  const handleChange = useCallback(async (newValue, ev) => {
    var _props$onChange;

    (_props$onChange = props.onChange) === null || _props$onChange === void 0 ? void 0 : _props$onChange.call(props, newValue, ev);

    if (!newValue) {
      return;
    }

    try {
      const schema = await setSchema(newValue);

      if (schema) {
        var _props$onSchemaChange;

        (_props$onSchemaChange = props.onSchemaChange) === null || _props$onSchemaChange === void 0 ? void 0 : _props$onSchemaChange.call(props, schema, newValue);
      }
    } catch (e) {
      if (!props.onSchemaError) {
        return;
      }

      const error = e instanceof GraphQLError ? e : new GraphQLError(e.message, undefined, undefined, undefined, undefined, e);
      props.onSchemaError([error], newValue, languageService);
    }
  }, [props.onChange, props.onSchemaChange, props.onSchemaError]);
  return /*#__PURE__*/jsx(MonacoEditor, {
    height: "70vh",
    theme: theme === 'dark' ? 'vs-dark' : 'light',
    ...props,
    beforeMount: handleBeforeMount,
    onMount: handleMount,
    onChange: handleChange,
    options: {
      glyphMargin: true,
      ...props.options
    },
    language: "graphql",
    defaultValue: props.defaultValue || props.schema
  });
}

const SchemaEditor = /*#__PURE__*/forwardRef(BaseSchemaEditor);

/*
 * `monaco-editor` throws an error while importing languages enums - Error: PostCSS plugin autoprefixer requires PostCSS 8
 *
 * import { languages } from 'monaco-editor'
 * languages.CompletionItemKind
 * languages.CompletionItemInsertTextRule
 *
 * So I hardcoded them inside this file
 */
let CompletionItemKind;

(function (CompletionItemKind) {
  CompletionItemKind[CompletionItemKind["Method"] = 0] = "Method";
  CompletionItemKind[CompletionItemKind["Function"] = 1] = "Function";
  CompletionItemKind[CompletionItemKind["Constructor"] = 2] = "Constructor";
  CompletionItemKind[CompletionItemKind["Field"] = 3] = "Field";
  CompletionItemKind[CompletionItemKind["Variable"] = 4] = "Variable";
  CompletionItemKind[CompletionItemKind["Class"] = 5] = "Class";
  CompletionItemKind[CompletionItemKind["Struct"] = 6] = "Struct";
  CompletionItemKind[CompletionItemKind["Interface"] = 7] = "Interface";
  CompletionItemKind[CompletionItemKind["Module"] = 8] = "Module";
  CompletionItemKind[CompletionItemKind["Property"] = 9] = "Property";
  CompletionItemKind[CompletionItemKind["Event"] = 10] = "Event";
  CompletionItemKind[CompletionItemKind["Operator"] = 11] = "Operator";
  CompletionItemKind[CompletionItemKind["Unit"] = 12] = "Unit";
  CompletionItemKind[CompletionItemKind["Value"] = 13] = "Value";
  CompletionItemKind[CompletionItemKind["Constant"] = 14] = "Constant";
  CompletionItemKind[CompletionItemKind["Enum"] = 15] = "Enum";
  CompletionItemKind[CompletionItemKind["EnumMember"] = 16] = "EnumMember";
  CompletionItemKind[CompletionItemKind["Keyword"] = 17] = "Keyword";
  CompletionItemKind[CompletionItemKind["Text"] = 18] = "Text";
  CompletionItemKind[CompletionItemKind["Color"] = 19] = "Color";
  CompletionItemKind[CompletionItemKind["File"] = 20] = "File";
  CompletionItemKind[CompletionItemKind["Reference"] = 21] = "Reference";
  CompletionItemKind[CompletionItemKind["Customcolor"] = 22] = "Customcolor";
  CompletionItemKind[CompletionItemKind["Folder"] = 23] = "Folder";
  CompletionItemKind[CompletionItemKind["TypeParameter"] = 24] = "TypeParameter";
  CompletionItemKind[CompletionItemKind["User"] = 25] = "User";
  CompletionItemKind[CompletionItemKind["Issue"] = 26] = "Issue";
  CompletionItemKind[CompletionItemKind["Snippet"] = 27] = "Snippet";
})(CompletionItemKind || (CompletionItemKind = {}));

let CompletionItemInsertTextRule;

(function (CompletionItemInsertTextRule) {
  CompletionItemInsertTextRule[CompletionItemInsertTextRule["KeepWhitespace"] = 1] = "KeepWhitespace";
  CompletionItemInsertTextRule[CompletionItemInsertTextRule["InsertAsSnippet"] = 4] = "InsertAsSnippet";
})(CompletionItemInsertTextRule || (CompletionItemInsertTextRule = {}));

var lsCIK;

(function (lsCIK) {
  lsCIK[lsCIK["Text"] = 1] = "Text";
  lsCIK[lsCIK["Method"] = 2] = "Method";
  lsCIK[lsCIK["Function"] = 3] = "Function";
  lsCIK[lsCIK["Constructor"] = 4] = "Constructor";
  lsCIK[lsCIK["Field"] = 5] = "Field";
  lsCIK[lsCIK["Variable"] = 6] = "Variable";
  lsCIK[lsCIK["Class"] = 7] = "Class";
  lsCIK[lsCIK["Interface"] = 8] = "Interface";
  lsCIK[lsCIK["Module"] = 9] = "Module";
  lsCIK[lsCIK["Property"] = 10] = "Property";
  lsCIK[lsCIK["Unit"] = 11] = "Unit";
  lsCIK[lsCIK["Value"] = 12] = "Value";
  lsCIK[lsCIK["Enum"] = 13] = "Enum";
  lsCIK[lsCIK["Keyword"] = 14] = "Keyword";
  lsCIK[lsCIK["Snippet"] = 15] = "Snippet";
  lsCIK[lsCIK["Color"] = 16] = "Color";
  lsCIK[lsCIK["File"] = 17] = "File";
  lsCIK[lsCIK["Reference"] = 18] = "Reference";
  lsCIK[lsCIK["Folder"] = 19] = "Folder";
  lsCIK[lsCIK["EnumMember"] = 20] = "EnumMember";
  lsCIK[lsCIK["Constant"] = 21] = "Constant";
  lsCIK[lsCIK["Struct"] = 22] = "Struct";
  lsCIK[lsCIK["Event"] = 23] = "Event";
  lsCIK[lsCIK["Operator"] = 24] = "Operator";
  lsCIK[lsCIK["TypeParameter"] = 25] = "TypeParameter";
})(lsCIK || (lsCIK = {}));

const toCompletionItemKind = kind => {
  const CIK = CompletionItemKind;
  const map = {
    [lsCIK.Text]: CIK.Text,
    [lsCIK.Method]: CIK.Method,
    [lsCIK.Function]: CIK.Function,
    [lsCIK.Constructor]: CIK.Constructor,
    [lsCIK.Field]: CIK.Field,
    [lsCIK.Variable]: CIK.Variable,
    [lsCIK.Class]: CIK.Class,
    [lsCIK.Interface]: CIK.Interface,
    [lsCIK.Module]: CIK.Module,
    [lsCIK.Property]: CIK.Property,
    [lsCIK.Unit]: CIK.Unit,
    [lsCIK.Value]: CIK.Value,
    [lsCIK.Enum]: CIK.Enum,
    [lsCIK.Keyword]: CIK.Keyword,
    [lsCIK.Snippet]: CIK.Snippet,
    [lsCIK.Color]: CIK.Color,
    [lsCIK.File]: CIK.File,
    [lsCIK.Reference]: CIK.Reference,
    [lsCIK.Folder]: CIK.Folder,
    [lsCIK.EnumMember]: CIK.EnumMember,
    [lsCIK.Constant]: CIK.Constant,
    [lsCIK.Struct]: CIK.Struct,
    [lsCIK.Event]: CIK.Event,
    [lsCIK.Operator]: CIK.Operator,
    [lsCIK.TypeParameter]: CIK.TypeParameter
  };
  return map[kind] || CIK.Text;
};

const toCompletion = entry => {
  return {
    range: entry.range,
    kind: toCompletionItemKind(entry.kind),
    label: entry.label,
    insertText: entry.insertText ?? entry.label,
    insertTextRules: entry.insertText ? CompletionItemInsertTextRule.InsertAsSnippet : undefined,
    sortText: entry.sortText,
    filterText: entry.filterText,
    documentation: entry.documentation,
    detail: entry.detail,
    command: entry.command
  };
};

class GraphQLWorker {
  doComplete(documentModel, position, schema) {
    const document = documentModel.getValue();

    if (!document) {
      console.log('no document');
      return [];
    }

    if (!schema) {
      console.log('no schema');
    }

    const graphQLPosition = toGraphQLPosition(position);
    const suggestions = getAutocompleteSuggestions(schema, document, graphQLPosition);
    return suggestions.map(suggestion => this.toCompletion(suggestion));
  }

  toCompletion(entry, range) {
    return {
      label: entry.label,
      insertText: entry.insertText,
      insertTextFormat: entry.insertTextFormat,
      sortText: entry.sortText,
      filterText: entry.filterText,
      documentation: entry.documentation,
      detail: entry.detail,
      range: range ? toMonacoRange(range) : undefined,
      kind: entry.kind,
      command: entry.command ? { ...entry.command,
        id: entry.command.command
      } : undefined
    };
  }

}

const ExecutableDocumentEditor = _ref => {
  let {
    schema,
    ...editorProps
  } = _ref;
  const {
    theme
  } = h();
  const monaco = useMonaco();
  const [completionProvider, setCompletionProvider] = useState(null);
  const editorUriRef = useRef();
  useEffect(() => {
    if (!monaco || !schema) {
      return;
    }

    if (completionProvider) {
      // dispose instance to prevent having duplicated field
      completionProvider.dispose();
    }

    const newProvider = monaco.languages.registerCompletionItemProvider('graphql', {
      triggerCharacters: [':', '$', '\n', ' ', '(', '@'],

      provideCompletionItems(model, position) {
        const isUriEquals = model.uri.path === editorUriRef.current.path;

        if (!isUriEquals) {
          return {
            suggestions: []
          };
        }

        const worker = new GraphQLWorker();
        const completionItems = worker.doComplete(model, position, schema);
        return {
          incomplete: true,
          suggestions: completionItems.map(item => toCompletion(item))
        };
      }

    });
    setCompletionProvider(newProvider);
  }, [monaco, schema]);
  return /*#__PURE__*/jsx(MonacoEditor, {
    height: "70vh",
    theme: theme === 'dark' ? 'vs-dark' : 'light',
    ...editorProps,
    language: "graphql",
    onMount: editor => {
      editorUriRef.current = editor.getModel().uri;
    }
  });
};

export { EnrichedLanguageService, ExecutableDocumentEditor, SchemaEditor, coreDefinitionSource, coreDiagnosticsSource, coreHoverSource, debugHoverSource, emptyLocation, locToRange, removeFalsey, showWidgetInPosition, toGraphQLPosition, toMarkerData, toMonacoRange };
//# sourceMappingURL=index.mjs.map
